package org.ow2.petals.se.pojo.listener;

import static org.ow2.petals.component.framework.util.ClassLoaderUtil.createClassLoader;

import java.util.Map;
import java.util.logging.Level;

import org.ow2.petals.component.framework.api.exception.FaultException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.se.pojo.Pojo;
import org.ow2.petals.se.pojo.PojoSE;

public class JBIListener extends AbstractJBIListener {

	@Override
	protected void onJBIMessage(final Exchange exchange) {
		final Map<String, Pojo> pojos = ((PojoSE) getComponent()).getPojos();
		Pojo pojo;
		synchronized (pojos) {
		    pojo = pojos.get(exchange.getEndpoint().getEndpointName());
    		if (pojo == null) {
    			try {
    				this.getLogger().info("POJO not loaded, create it.");
    				pojo = this.createPojo();
    				pojos.put(exchange.getEndpoint().getEndpointName(), pojo);
    			} catch (final Exception e) {
    				this.getLogger().severe(e.getMessage());
    				exchange.setError(e);
    			}
    		}
		}
        
		if (pojo != null){
		    try {
    			try {
    				pojo.callOnExchangeMethod(exchange);
    			} catch (final FaultException e) {
                    this.getLogger().log(Level.FINE, e.getMessage(), e);
                    exchange.setFault(e);
                }
		    } catch (final Exception e) {
				this.getLogger().log(Level.SEVERE, e.getMessage(), e);
				exchange.setError(e);
			}
		}
	}

	private Pojo createPojo()
			throws Exception {
		// retrieve pojo' class name
		String className = null;
		className = getExtensions().get("class-name");
		if (className == null) {
			throw new Exception(
					"'class-name' property not found in the JBI.XML descriptor");
		}

		// create a classloader from the SU path
		ClassLoader classLoader = createClassLoader(this.getInstallRoot(), this.getClass()
				.getClassLoader());

		// load the pojo class from the classloader
		Class<?> clazz = null;
		try {
			clazz = classLoader.loadClass(className);
		} catch (ClassNotFoundException e) {
			throw new Exception(className
					+ " class not found in the service-unit libraries.", e);
		}

		// create the pojo
		Pojo pojo = null;
		try {
			pojo = new Pojo(clazz.newInstance(), classLoader, getComponent()
					.getContext(), getComponent().getChannel(), this, getComponent()
					.getLogger());
		} catch (Exception e) {
			throw new Exception(className
					+ " found, but can not create an instance.", e);
		}

		// init the pojo
		pojo.init();
		
		// start the pojo
		pojo.start();

		return pojo;
	}
}
